

// Sketch developed with VisualStudio 2013 and Visual Micro plugin for Arduino
//		http://visualmicro.codeplex.com/downloads/get/822060
//		http://www.visualmicro.com/
// 
// This sketch was based on the ServoActuator.ino sketch from the mySensor samples
//
// ServoActuator.ino
// Example showing how to create an actuator for a servo.
// Connect red to +5V, Black or brown to GND and the last cable to Digital pin 3.
// The servo consumes much power and should probably have its own powersource.'
// The arduino might spontanally restart if too much power is used (happend
// to me when servo tried to pass the extreme positions = full load).
// Contribution by: Derek Macias
//
// July 2014 Mike Lininger
// For use with mySensors 1.4b+ 
// This project will allow the automatation of a window blind (1" Faux Wood Style) tilting feature,
// via a VeraLite, using vera scenes with Day/Night detection.
//
// This was my learning project to make use of wonderful software library of the mySensors project
// that allows users of the VERA automation system to interface with home made sensors using the
// arduino platform. http://www.mysensors.org/

// See attached wiring schematic for assembling the sensor controller.

// Notes:  Your servo may behave erratically, This is because the servo draws quite a lot of power, 
//         especially as the motor is starting up, and this sudden high demand can be enough to drop 
//         the voltage on the Arduino board, so that it resets itself.  If this happens, then you can 
//         usually cure it by adding a high value capacitor (470uF or greater) between GND and 5V on the breadboard. 
//         or power the servo from a seperate source. When you power the servo from a separate power source you will
//         have to connect all of the grounds together. 
//         https://learn.adafruit.com/adafruit-arduino-lesson-14-servo-motors/if-the-servo-misbehaves
//         http://rcarduino.blogspot.com/2012/04/servo-problems-with-arduino-part-1.html      

//         Hardware List
//         1 - Arduino Mini Pro v3.3
//         1 - 220uF Capacitor (NRF24 across 3v and GND 16v+ rating)
//         1 - 470uF Capacitor Optional (Servo across 5v and GND 16v+ rating)
//         1 - Servo (Micro 25.0 oz-in [1.80 kg-cm] minimum) for selectblinds.com 1" Faux Wood Blinds 43"x38"
//         1 - NRF24L01 Wireless Transceiver Module (flat or antenna version) Be aware of power limitations when using the NRF24L01 antenna with LNA+PA 
//             See for discussions about this http://forum.mysensors.org/topic/168/arduino-library-1-4b1-call-for-beta-testers#1850 around message #105
//         1 - Slide switch (SPST or DPDT) or Push Button
//         1 - 120k Ohms 5% Resister (Brown-Black-Orange-Gold) to pull down switch or button so it does not float around
//         1 - Kimbrough Internal Rotary Drive Control System 500 NIB - connects servo to blind rod (may differ for you blind type and servo used) 
//             http://www.ebay.com/itm/291084936296?ssPageName=STRK:MEWAX:IT&_trksid=p3984.m1423.l2649
//         1 - DC power supply You may need to step-up or step-down for the servo 5v and 3.3v for the NRF depending on the Arduino board you use
//             http://www.ebay.com/itm/321408108698?ssPageName=STRK:MEWNX:IT&_trksid=p3984.m1439.l2649 is the one I used to power arduino and servo seperatly

// Included a seperate debug process that can be enabled and disabled without conflicting
// with the base mySensor DEBUG (that is located in the config.h file).  The mySensor library
// has been well debugged and we don't need to see their debug info if we would share their debug routine.

// Include the various mySensor library files (1.4b build 18848a2)
#include <MySensor.h>  
#include <SPI.h>
#include <Servo.h> 
                                                                                      // enable/disable project only debug logging 
#define PDEBUG 1                                                                      // comment this line for no debug printing when project is ready for deployment will reduce memory as I dump a lot of info
                                                                                      // to the serial output window, so new users can see how the various pieces interact with one another                                                                                   
                                                                                      // Define the Arduino Pins to be used in this project                                                                                        
#define SERVO_DIGITAL_OUT_PIN 3                                                       // digital for Servo motor (simulate PWM)                                            
#define BUTTON_DIGITAL_IN_PIN 4                                                       // digital for Override switch/Button (on/off) - Force blinds closed

#define MAX_SEND_RETIRES 20                                                           // sending data backup to the GW does not always succeed on first, second ... attempts. Try this many times before we give up
#define PERCENT_MIN 0                                                                 // fine tune your allowed minimum percentage value (usually 0)
#define PERCENT_MAX 100                                                               // fine tune your allowed maximum percentage value (usually 100)
#define SERVO_MIN 0                                                                   // fine tune your servos min. 0-180 for cheaper servo might want to use 10  - they have problems remembering
#define SERVO_MAX 180                                                                 // fine tune your servos max. 0-180 for cheaper servo might want to use 170 - their positions around 0 and 180
#define DETACH_DELAY 1000                                                             // tune this to let your motor movement finish before detaching the servo
#define CHILD_SERVO_ID 40                                                             // id of the sensor servo child
#define powerSensorInterval 900000                                                    // number of seconds to wait until the next power level check 
#define baseMilliVoltageLevel 3300                                                    // voltage supplied from the arduino board VIN
                                                                                      // 5.0  * 1000  = 5000  
                                                                                      // 3.3  * 1000  = 3300
                                                                                      
MySensor gw;                                                                          // create sensor object
Servo myservo;                                                                        // create servo object to control the servo 

MyMessage dimmerMsg(CHILD_SERVO_ID, V_DIMMER);                                        // create a message object (dimmerMsg) so we can communicate with the gateway/vera using the dimmer bar
MyMessage variable2Msg(CHILD_SERVO_ID, V_VAR2);                                       // create a message object (variable2Msg) so we can communicate with the gateway/vera variable2 (under advanced)

unsigned long timeOfLastVoltageChange = 0;                                            // holds the time in milliseconds when the battery level was last checked
unsigned long lastPowerPercentage = 0;                                                // holds the last mapped % of the battery
unsigned long timeOfLastServoChange = 0;                                              // holds the time in milliseconds when the servo last changed position
                                                                                      // allows for the detachment of the servo after a small delay (see DETACH_DELAY) so
                                                                                      // power is not constantly being applied to the servo prevents battery, if used, 
                                                                                      // from being drained
                                                                                        
int buttonState = 0;                                                                  // variable for reading the pushbutton status								          
bool attachedServo = false;                                                           // tracks whether or not the servo is currentl attached            
bool button_override_state = false;                                                   // tracks whether the override button is pressed	
                                                                                      // setup the debugging process
                                                                                      // if PDEBUG is defined (not removed) then stream debug data to the serial window.
#ifdef PDEBUG                                                                                     
  void StreamPrint_progmem(Print &out,PGM_P format,...)
  {                                                                                   // http://www.utopiamechanicus.com/article/low-memory-serial-print/
                                                                                      // program memory version of printf - copy of format string and result share a buffer
                                                                                      // so as to avoid too much memory use. There is no println() version – instead, 
    char formatString[128], *ptr;                                                     // add \n or \r to your strings
    strncpy_P( formatString, format, sizeof(formatString) );                          // copy in from program mem
                                                                                      // null terminate - leave last char since we might need it in worst case for result's \0
    formatString[ sizeof(formatString)-2 ]='\0'; 
    ptr=&formatString[ strlen(formatString)+1 ];                                      // our result buffer...
    va_list args;
    va_start (args,format);
    vsnprintf(ptr, sizeof(formatString)-1-strlen(formatString), formatString, args );
    va_end (args);
    formatString[ sizeof(formatString)-1 ]='\0'; 
    out.print(ptr);
  } 
  #define pdebug(format, ...) StreamPrint_progmem(Serial,PSTR(format),##__VA_ARGS__)
#else
  #define pdebug(x,...)                                                                // do nothing if the #define PDEBUG 1 is commented out
#endif

void setup() 
{ 
  delay(2000);                                                                         // allow delay on startup in case this start is the result of a power failure as
                                                                                       // all other nodes will also be sending/requestion info from the GW
  gw.begin(incomingMessage);                                                           // attach callback method for incoming messages

                                                                                       pdebug("outbound: Sending Sketch Info\n"); 
  gw.sendSketchInfo("Blinds", "1.0", true);                                            // send the sketch version information to the gateway and Controller
                                                                                       pdebug("outbound: Sending Presentation Info\n"); 
  gw.present(CHILD_SERVO_ID, S_COVER, true);                                           // register all sensors to gw (will be created as child device)

                                                                                       // fetch servo state from the vera using the current dimmer percentage at startup
                                                                                       // so we know how the blinds should be tilted when the sensor first boots up.
                                                                                       // example:  I default to a value of 0 (CLOSED).  This value can be anything
                                                                                       // between 0 (CLOSED) to 50 (FULL OPEN) 100 (CLOSED) (My Binds have 180 degrees
                                                                                       // of tilt form closed [0] to open [90] back to closed [180] (SEE BLIND NOTES BELOW)
                                                                                       // if power is lost at night, I don't want the blinds going into a open state
                                                                                       // as the relay initializes with the return of power.  Note you could also store in the procedure below.
                                                                                       // EEPROM, but writes are limited in quantity. Reply should appear in incomingMessage   
																					                                                                                      
  gw.request(CHILD_SERVO_ID, V_DIMMER);                                                // fetch last dimmer postion state at startup (Dimmer Position from vera object)
                                                                                       // if debugging send data to serial monitor.  Reply should appear in incomingMessage
                                                                                       // procedure below.
                                                                                       pdebug("outbound: Request Dimmer Position from vera object %d\n", V_DIMMER);                                                                                                                                                                            
  pinMode(BUTTON_DIGITAL_IN_PIN, INPUT);                                               // initialize the pushbutton pin as an input:
} 


void loop() 
{ 
  gw.process();                                                                         // call this often to check from any incomming messages from the vera.  This node
                                                                                        // runs off AC power so no sleeping will happen because we are expecting instructions
                                                                                        // to be sent by the vera-GW module  
  buttonState = digitalRead(BUTTON_DIGITAL_IN_PIN);                                     // read the state of the push button value:   
                                                                                        // have we used the servo resently.  If so has enough time passed after sending
  if (attachedServo && millis() - timeOfLastServoChange > DETACH_DELAY)                 // the last position, go ahead and detach the servo, so we are not consumming power
   {
     myservo.detach();                                                                  // detach the servo
     attachedServo = false;                                                             // set local detachment state of the servo object
     button_override_state = false;  
                                                                                        pdebug("local: servo detached %d\n", attachedServo); 
   }  
                                                                                        // check if the switch is in the override position.                                                                                     
  if (buttonState == HIGH) {                                                            // if it is, the buttonState is HIGH:
       if (button_override_state == false) {
          myservo.attach(SERVO_DIGITAL_OUT_PIN);                                        // we have an action to perform so get the servo ready																				   
          button_override_state = true;                                                 // switch is in the closed state, closing the blinds
          timeOfLastServoChange = millis();                                             // allow time for the servo to finish it movement
          attachedServo = true;                                                         pdebug("local: servo attached %d\n", attachedServo); 
                                                                                        pdebug("local: OVERRIDE Button/Switch is in closed state\n");                                                                                        
          myservo.write(SERVO_MIN);                                                     // move the servo to full CLOSED [0] position
	  sendWithConfirmation(dimmerMsg, PERCENT_MIN);                                 // let the Gateway (GW) and vera know that we changed to 100% closed  
                                                                                        pdebug("local: Servo (mapped) value %d\n", SERVO_MIN); 
       }
   } 
 
                                                                                        // check power level every x seconds (defined in the powerSensorInterval constant)                         
   if (millis() - timeOfLastVoltageChange > powerSensorInterval)  {                                                                                       
      long estimatedInternalVoltage = getInternalVoltage();		                                                                             
      int powerPercentage = map(estimatedInternalVoltage, 0, baseMilliVoltageLevel, 0, 100);                                 
                                                                                        // take the milli-volt reading and map it to a 0 to 100% percentage based on what we expect
      if (powerPercentage > 100) {
        powerPercentage = 100;                                                          // has to between 0 and 100
      }
      else if (powerPercentage < 0) {
          powerPercentage = 0;                                                          // has to between 0 and 100
      }
      	                                                                                
      if (powerPercentage != lastPowerPercentage) {
                                                                                        pdebug("local: Current power level (percentage) is %d\n", powerPercentage);  
        gw.sendBatteryLevel(powerPercentage, true);                                     // send the percentage value back to the Gateway (GW) so vera can show the battery level does not return success/fail boolean
                                                                                        pdebug("outbound: Sending to Variable2 (milli-volts) is %d\n", estimatedInternalVoltage); 
	sendWithConfirmation(variable2Msg, estimatedInternalVoltage);                   // let the Gateway (GW) and vera know that we changed                                                                  
        lastPowerPercentage = powerPercentage;                                          // set the new battery percentage.
      } 	                                                                                
      timeOfLastVoltageChange = millis();                                               // reset last time voltage was changed
   }
} 

                                                                                        // this callback process handles messages/responses received from the Vera Gateway
void incomingMessage(const MyMessage &message)
{ 
                                                                                        pdebug("-----: new inbound message arrived\n");
                                                                                        // determine what type of message from Gateway (GW) we need to react to
                                                                                        pdebug("inbound: message.type %d\n", message.type); 
                                                                                        // Note: Close and Open buttons of the vera window covering node send a V_Dimmer of 0 and 100 percent when pressed
  if (message.type==V_DIMMER)                                                           
  {																						
                                                                                        pdebug("inbound: V_DIMMER (slider) notification\n");
    int val = message.getInt();                                                         // we are dealing with V_DIMMER slider change of position
                                                                                        // 0   = Closed                               [0 degrees]
                                                                                        // #   = Some Percent Open between 1 and 49   [1-89 degrees]
                                                                                        // 50  = Open  (SEE BLIND NOTES BELOW)        [90 degrees]
                                                                                        // #   = Some Percent Open between 51 and 99  [91-179 degrees]
                                                                                        // 100 = Closed                               [180 degrees]
                                                                                        pdebug("inbound: V_DIMMER (slider) value %d\n", val);
    if (attachedServo == false) {
        attachedServo = true;                                                           // set local attachment state of the servo object
        myservo.attach(SERVO_DIGITAL_OUT_PIN);                                          // we have an action to perform so get the servo ready
	}
                                                                                        pdebug("local: servo attached %d\n", attachedServo); 
                                                                                        // here we take a value between 0% and 100% from the Gateway (GW)   
    int val_map = map(val, PERCENT_MIN, PERCENT_MAX, SERVO_MIN, SERVO_MAX);             // and map it to a value suitable for the servo (0 to 180 degrees of rotation
    timeOfLastServoChange = millis();                                                   // mark the last time we changed the servo position so we can detach it after a small delay
    myservo.write(val_map);                                                             // move the servo to new mapped position	
                                                                                        pdebug("local: servo (mapped) value %d\n", val_map); 
  } 
  else if (message.type==V_UP) 
  {
                                                                                        // we are dealing with V_UP button being clicked
                                                                                        pdebug(" inbound: V_UP (button) notification\n");                                                                                   
    int val_map = map(PERCENT_MAX, PERCENT_MIN, PERCENT_MAX, SERVO_MIN, SERVO_MAX);     // we treat V_UP as close the blind 100%
    sendWithConfirmation(dimmerMsg, PERCENT_MAX);					// let the Gateway (GW) and vera know that we changed to 100% closed  
	                                                                                // Vera-Gateway will echo back the V_DIMMER message so the servo can be set to the proper position
                                                                                        pdebug("outbound: V_DIMMER (slider) value %d\n", PERCENT_MAX); 
  } 
  else if (message.type==V_DOWN) 
  {
                                                                                        // we are dealing with V_DOWN button being clicked
                                                                                        pdebug(" inbound: V_DOWN (button) notification\n");
                                                                                        // we treat V_DOWN as close the blind 0%
    sendWithConfirmation(dimmerMsg, PERCENT_MIN);					// let the Gateway (GW) and vera know that we changed to 0% closed
	                                                                                // Vera-Gateway will echo back the V_DIMMER message so the servo can be set to the proper position
	                                                                                pdebug("outbound: V_DIMMER (slider) value %d\n", PERCENT_MIN); 
  } 
  else if (message.type==V_STOP) 
  {
                                                                                        pdebug("inbound: V_STOP (button) notification\n"); 
                                                                                        // Don't really need to do anything will detach itself after DETACH_DELAY  
  } 
  else
  {     
                                                                                        pdebug("inbound: un-handled message.type received\n");
  }
                                                                                        pdebug("-----: done pocessing message\n");
}

void sendWithConfirmation(MyMessage &msg, int send_value)
{
	bool confirmation = false;
	int maxRetry = MAX_SEND_RETIRES;						// determine how my attempt we should make to send the message back to the gateway
    do {
      confirmation = gw.send(msg.set(send_value), true);				// Did the message make it successfully to the first node in the possible chain of nodes back to the gateway
	  if (!confirmation) {								// true = we got an ACK from the node, false = we got no response so lets try again
            gw.process();							        // While attempt to send message also check for any new incomming messages, don't want to miss them
	  }
	                                                                                pdebug("local: gw.send message status %d attempt: %d duration: %lu\n", confirmation, (MAX_SEND_RETIRES-maxRetry), millis()); 
    } while ( !confirmation && (--maxRetry > 0));
	if (maxRetry == 0) {
											pdebug("local: gw.send FAILED for message.type: %d value: %d\n", msg.type, send_value);
	}
	else {
											pdebug("outbound: gw.send SUCCESSFUL for message.type: %d value: %d\n", msg.type, send_value);
	}
}
                                                                                        // this method was pulled from http://www.mysensors.org/git/Arduino/libraries/MySensors/examples/BatteryPoweredSensor/BatteryPoweredSensor.ino
                                                                                        // and included in this sketch 
long getInternalVoltage()
{
                                                                                        // this is an example that demonstrates how to report the battery level for a sensor
                                                                                        // instructions for measuring battery capacity on A0 are available in the follwoing forum
                                                                                        // thread: http://forum.micasaverde.com/index.php/topic,20078.0.html
  long result;
  ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);                               // read 1.1V reference against AVcc
  delay(2);                                                                             // wait for Vref to settle
  ADCSRA |= _BV(ADSC);                                                                  // convert
  while (bit_is_set(ADCSRA,ADSC));
  result = ADCL;
  result |= ADCH<<8;
  result = 1125300L / result;                                                           // back-calculate AVcc in mV
  return result;
}
 
